(* ::Package:: *)

(*October 20 2022*)
(*August (Young) Frechette*)


BeginPackage["HDPWolframSolverModules`"]
Unprotect @@ Names["HDPWolframSolverModules`*"];
ClearAll @@ Names["HDPWolframSolverModules`*"];


FlowSolver::usage = "This package generates a streamline plot in the dead-end pore corresponding to the input geometry and Reynolds Number";

SepPointPlot::usage ="This package requires the user to approximate the location of the separatrix";

InterFun::usage ="This package uses the output from the SepPointPlot Module to generate a mathematical description of the separatrix";


Begin["`Private`"]


Needs["NDSolve`FEM`"]

FlowSolver[RE_,h1_,h2_,gs_] := Module[
{u, v, p, x, y, mesh, RR, height, chanHeight, length, depth, DEP, DEPnd, mrf, U, PDiff, PDiffND, OP, EQs, BCs, bcVelocity, bcWalls, bcOutlet},

length = 4.*height;(*m*)
depth =height;
DEP=RegionUnion[Rectangle[{0.,0.},{length,height}],Rectangle[{height,0},{2.*height,-depth}]];
DEPnd=RegionUnion[Rectangle[{0.,0.},{length,height}/height],Rectangle[{height,0.}/height,{2.*height,-depth}/height]];
Clear[x,y];
mesh = ToElementMesh[DEPnd,MaxCellMeasure->0.012,"MaxBoundaryCellMeasure"->0.01,MeshQualityGoal ->"Maximal"];
RR=Rectangle[{1*height, h1}/height, {2*height,h2}/height ];
mrf=With[{rmf=RegionMember[RR]},Function[{vertices,area},Block[{a,b},{a,b}=Mean[vertices];
If[rmf[{a,b}],area>0.000025,area>0.000025]]]];

\[Mu] = 1.23*10^-3;(*kg m^-1s^-1*)
\[Rho] = 9.99*10^2;(*kg m^-3*)

U =(RE \[Mu])/(\[Rho] height);
PDiff = -12*\[Mu]*U/height^2; (*N/m^3*)
PDiffND =-12/RE;

OP = {{{u[x,y],v[x,y]}} . Inactive[Grad][u[x,y],{x,y}]+D[p[x,y],x] -RE^-1*Laplacian[u[x, y],{x, y}] ,
{{u[x,y],v[x,y]}} . Inactive[Grad][v[x,y],{x,y}]+D[p[x,y],y] -RE^-1*Laplacian[v[x, y],{x, y}] ,
D[u[x,y],x]+D[v[x,y],y]};
EQs = OP == {0.,0.,0.};

bcWalls = DirichletCondition[{u[x,y]==0.,v[x,y]==0.}, x>0. && x<length/height];
bcVelocity = DirichletCondition[{u[x,y]== RE/2 PDiffND(y^2-y),v[x,y]==0.},x==0.];
bcOutlet = DirichletCondition[p[x,y]==0.,x==length/height];
BCs ={bcVelocity,bcWalls, bcOutlet };

Clear[u,v,p];
{xVel, yVel, pressure} = NDSolveValue[{EQs, BCs},{u,v,p},{x,y}\[Element]mesh,Method->{"FiniteElement","InterpolationOrder"->{u->2,v->2,p->1},"MeshOptions"->{AccuracyGoal->16,PrecisionGoal->16,MeshRefinementFunction ->mrf}}];

GridPointsDEP=Tuples[{Range[1*height/height,2*height/height,gs],Range[h1*height/height,h2*height/height,gs]}];
{DEPnd,StreamPlot[{xVel[x,y],yVel[x,y]},{x,y} \[Element] DEPnd, 
PlotRange->{{1,2}*height/height,{h1,h2}*height/height},AspectRatio->Automatic,StreamPoints->{GridPointsDEP,Fine,Scaled[2]},StreamStyle->Lighter[ColorData[97,"ColorList"][[1]],0.15],StreamScale->None,StreamColorFunction->None,PlotRangePadding->None,Frame->False]}
]


InterFun[X_,h1_,h2_,IO_]:= Module[{Xpts,Ypts,Points},

Xpts = X[[All,1]]; Ypts = X[[All,2]];
Points = Transpose[{Xpts,Ypts}];
ifun =Interpolation[Points, InterpolationOrder->IO];
{Plot[ifun[x],{x, 1, 2},Epilog->Point[Points],AspectRatio->Automatic,PlotRange->{h1,h2}],ifun}
]


End[]


SepPointPlot[img_,h1_,h2_]:=DynamicModule[
{pts={{(1+1/6),(h1+h2)/2},{(1+2/6),(h1+h2)/2},{(1+3/6),(h1+h2)/2},{(1+4/6),(h1+h2)/2},{(1+5/6),(h1+h2)/2}}},

Grid[{{LocatorPane[Dynamic[pts],Dynamic[Show[Plot[InterpolatingPolynomial[pts,z],{z,1,2},PlotRange->{{1,2},{h1,h2}},PlotStyle->{Directive[Red,Thickness[0.0035]]},ImageSize->Large,Frame->True],img]],LocatorAutoCreate->True],
Button["SAVE",data=pts,BaseStyle->{"GenericButton",14,Bold},ImageSize->{100,45}]}}]]


Unprotect @@ Names["HDPWolframSolverModules`*"];
EndPackage[]
